#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""PAL command for USER configuration."""

import os
import pal_config as config
from iobase.exception import (ExecError, InvalidArgError)
from iobase.cmdexec import CmdExec, NULL, PIPE
from iobase import xmldata
from ionas.request import Entry
from libpal import palcmd
from libpal.crond import ctrl as crond
from pal_info import (ReqTypes, Results)
from subprocess import Popen, PIPE
from albg_info import (ALBG_CLONE_PATH,)


class PalClone(palcmd.PalCmd):
    """PAL Users command implementation class."""

    def __empty(self, val):
        if val in (None, "", [], (), {}):
            return True
        return False

    def __apply_handler(self, entry):
        """Handle apply request."""

        try:
            entry = entry.get_clone()
            clone_node = entry.get()

            def _is_enabled(node, xpath='.', default=False):
                try:
                    return node.get_node(xpath).is_enabled()
                except:
                    return default

            if clone_node.get_node_list('./master'):
                pass
            if clone_node.get_node_list('./slave'):
                pass
            if clone_node.get_node_list('./system'):
                cronname = 'SystemBackup'
                cronnamefull = 'SystemBackupFull'
                crond.delete_entry(cronname)
                crond.delete_entry(cronnamefull)
                if _is_enabled(clone_node, './system/schedule'):
                    hour = clone_node.get_value(
                            './system/schedule/time/hour', '*')
                    minute = clone_node.get_value(
                            './system/schedule/time/minute', '*')
                    full, diff = [], []
                    for day_node in [node for node in clone_node.get_node_list(
                            './system/schedule/day')]:
                        if day_node.is_set_to_true('full'):
                            full.append(day_node.get_text())
                        else:
                            diff.append(day_node.get_text())
                    cron = lambda name, full, week: crond.add_entry(
                            name,
                            """echo '<request><entrylist><entry type="backup"><clone><config enable="true" /><share enable="true" full="%(full)s" /></clone></entry></entrylist></request>' | /mnt/data/bin/fal_clone.py""" % {
                                'full': str(full).lower(),
                            },
                            user='root',
                            week=week,
                            hour=hour,
                            minutes=minute,
                        )
                    if diff:
                        cron(cronname, False, ','.join(diff))
                    if full:
                        cron(cronnamefull, True, ','.join(full))

            entry.set_result(Results.OK)
        except:
            entry.set_result(Results.INTERNAL_ERROR)
            self.exception("Error")  # FIXME message

        return entry

    def __send_handler(self, entry):
        """Handle send request.
            <request>
                <entrylist>
                    <entry type="send">
                        <clone>
                            <slave>
                                <host>landisk-123456</host>
                                <passwd>ioIO10!!</passwd>
                            </slave>
                        </clone>
                    </entry>
                </entrylist>
            </request>
        """

        try:
            # NOTE albg_clone を呼ぶ
            if (Results.OK == self.send_entry(entry,
					ALBG_CLONE_PATH).get_result()):
                entry.set_result(Results.OK)
            else:
                entry.set_result(Results.INTERNAL_ERROR)

        except ExecError:
            entry.set_result(Results.INTERNAL_ERROR)
            self.exception("Error")  # FIXME message

        return entry

    def __alive_handler(self, entry):
        """
        Check alive master.
        <entry type="alive">
            <clone>
                <target>(HOSTNAME|IPv4|IPv6)</target>
            </clone>
        </entry>
        """

        class Results(object):
            OK = "0"
            ERR = "1"
            ERR_DEAD = "2"
            ERR_TARGET_IS_REQUIRED = "3"

        try:
            entry = entry.get_clone()
            clone_node = entry.get()

            target = clone_node.get_value('target', None)
            if target is None:
                entry.set_result(Results.ERR_TARGET_IS_REQUIRED)
                raise

            ALIVE_CMD = '/usr/local/bin/landisk-alive'  # FIXME path
            proc = Popen([ALIVE_CMD, target])
            proc.communicate()
            alive = proc.returncode == 0
            clone_node.get_node('target').set_bool_attr('alive', true=alive)

            entry.set_result(Results.OK if alive else Results.ERR_DEAD)
        except:
            try:
                entry.get_result()
            except:
                entry.set_result(Results.ERR)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        palcmd.PalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.SEND, self.__send_handler)

        self.set_handler('alive', self.__alive_handler)

#
# Main
#

if __name__ == "__main__":
    PalClone(option=palcmd.PalCmdOption()).handle_request()
